@@ -2,10 +2,15 @@
 
 mod assign_id;
 mod lint;
+mod list_affected_versions;
+mod osv;
 mod version;
 mod web;
 
-use self::{assign_id::AssignIdCmd, lint::LintCmd, version::VersionCmd, web::WebCmd};
+use self::{
+    assign_id::AssignIdCmd, lint::LintCmd, list_affected_versions::ListAffectedVersionsCmd,
+    osv::OsvCmd, version::VersionCmd, web::WebCmd,
+};
 use crate::config::AppConfig;
 use abscissa_core::{Command, Configurable, Help, Options, Runnable};
 use std::path::PathBuf;
@@ -32,6 +37,14 @@ pub enum AdminCmd {
     /// The `assign-id` subcommand
     #[options(help = "assigning RUSTSEC ids to new vulnerabilities")]
     AssignId(AssignIdCmd),
+
+    /// The `osv` subcommand
+    #[options(help = "export advisories to OSV format")]
+    Osv(OsvCmd),
+
+    /// The `version` subcommand
+    #[options(help = "list affected crate versions")]
+    ListAffectedVersions(ListAffectedVersionsCmd),
 }
 
 impl Configurable<AppConfig> for AdminCmd {
